@@ -6,7 +6,7 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Stack.scala 9110 2006-11-01 16:03:28Z mihaylov $
+// $Id: Stack.scala 9980 2007-02-13 22:06:58Z odersky $
 
 
 package scala.collection.mutable
@@ -25,7 +25,7 @@ class Stack[A] extends MutableList[A] {
    *
    *  @return true, iff there is no element on the stack
    */
-  override def isEmpty: Boolean = (first == null)
+  override def isEmpty: Boolean = (first eq null)
 
   /** Pushes a single element on top of the stack.
    *
@@ -60,18 +60,18 @@ class Stack[A] extends MutableList[A] {
    *  the element from the stack. An error is signaled if there is no
    *  element on the stack.
    *
-   *  @throws scala.compat.Platform.NoSuchElementException
+   *  @throws Predef.NoSuchElementException
    *  @return the top element
    */
-  def top: A = if (first == null) throw new NoSuchElementException("stack empty") else first.elem
+  def top: A = if (first eq null) throw new NoSuchElementException("stack empty") else first.elem
 
   /** Removes the top element from the stack.
    *
-   *  @throws scala.compat.Platform.NoSuchElementException
+   *  @throws Predef.NoSuchElementException
    *  @return the top element
    */
   def pop: A =
-    if (first != null) {
+    if (first ne null) {
       val res = first.elem
       first = first.next
       len = len - 1;
@@ -83,7 +83,7 @@ class Stack[A] extends MutableList[A] {
    * Removes all elements from the stack. After this operation completed,
    * the stack will be empty.
    */
-  def clear: Unit = reset
+  def clear(): Unit = reset
 
   /** Returns an iterator over all elements on the stack. This iterator
    *  is stable with respect to state changes in the stack object; i.e.
