@@ -6,7 +6,7 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Platform.scala 9116 2006-11-02 08:46:39Z mihaylov $
+// $Id: Platform.scala 9896 2007-02-06 16:06:34Z mihaylov $
 
 
 package scala.compat;
@@ -17,24 +17,21 @@ import Predef.Class
 
 object Platform {
 
-  type StackOverflowError = java.lang.StackOverflowError;
+  type StackOverflowError = java.lang.StackOverflowError
 
   def arraycopy(src: AnyRef, srcPos: Int, dest: AnyRef, destPos: Int, length: Int): Unit =
     System.arraycopy(src, srcPos, dest, destPos, length)
 
   /** create array of the same type as arrayInstance with the given length */
   def createArray(elemClass: Class, length: Int): AnyRef =
-    java.lang.reflect.Array.newInstance(elemClass, length);
+    java.lang.reflect.Array.newInstance(elemClass, length)
 
-  def getClass(obj: AnyRef) = obj.getClass();
-  def getClassName(obj: AnyRef) = obj.getClass().getName();
-  def getName(cls: Class) = cls.getName();
-  def getElementClass(obj: AnyRef) = obj.getClass().getComponentType();
-  def getClassForName(name: String): Class = java.lang.Class.forName(name);
+  def getClassForName(name: String): Class = java.lang.Class.forName(name)
 
   val EOL = System.getProperty("line.separator", "\n")
 
   def currentTime: Long = System.currentTimeMillis()
+
   def collectGarbage: Unit = System.gc()
 
 }
