@@ -6,7 +6,7 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: BoxedIntArray.scala 9110 2006-11-01 16:03:28Z mihaylov $
+// $Id: BoxedIntArray.scala 10044 2007-02-20 11:23:17Z odersky $
 
 
 package scala.runtime
@@ -19,10 +19,10 @@ final class BoxedIntArray(val value: Array[Int]) extends BoxedArray {
 
   def length: Int = value.length
 
-  def apply(index: Int): AnyRef = BoxedInt.box(value(index))
+  def apply(index: Int): Any = Int.box(value(index))
 
-  def update(index: Int, elem: AnyRef): Unit = {
-    value(index) = elem.asInstanceOf[BoxedNumber].intValue()
+  def update(index: Int, elem: Any): Unit = {
+    value(index) = Int.unbox(elem.asInstanceOf[AnyRef])
   }
 
   def unbox(elemTag: String): AnyRef = value
@@ -38,9 +38,9 @@ final class BoxedIntArray(val value: Array[Int]) extends BoxedArray {
     val result = new Array[Int](end - start)
     Array.copy(value, start, result, 0, end - start)
     result
-  }    
+  }
 
-  def filter(p: Any => Boolean): Array[Int] = {
+  final override def filter(p: Any => Boolean): BoxedArray = {
     val include = new Array[Boolean](value.length)
     var len = 0
     var i = 0
@@ -55,7 +55,13 @@ final class BoxedIntArray(val value: Array[Int]) extends BoxedArray {
       if (include(i)) { result(len) = value(i); len = len + 1 }
       i = i + 1
     }
-    result
+    new BoxedIntArray(result)
+  }
+
+  final override def slice(start: Int, end: Int): BoxedArray = {
+    val len = end - start
+    val result = new Array[Int](len)
+    Array.copy(value, start, result, 0, len)
+    new BoxedIntArray(result)
   }
 }
-
