@@ -6,7 +6,7 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: BoxedShortArray.scala 9110 2006-11-01 16:03:28Z mihaylov $
+// $Id: BoxedShortArray.scala 10044 2007-02-20 11:23:17Z odersky $
 
 
 package scala.runtime;
@@ -19,10 +19,10 @@ final class BoxedShortArray(val value: Array[Short]) extends BoxedArray {
 
   def length: Int = value.length
 
-  def apply(index: Int): AnyRef = BoxedShort.box(value(index))
+  def apply(index: Int): Any = Short.box(value(index))
 
-  def update(index: Int, elem: AnyRef): Unit = {
-    value(index) = elem.asInstanceOf[BoxedNumber].shortValue()
+  def update(index: Int, elem: Any): Unit = {
+    value(index) = Short.unbox(elem.asInstanceOf[AnyRef])
   }
 
   def unbox(elemTag: String): AnyRef = value
@@ -33,14 +33,14 @@ final class BoxedShortArray(val value: Array[Short]) extends BoxedArray {
     other.isInstanceOf[BoxedShortArray] && value == other.asInstanceOf[BoxedShortArray].value
 
   override def hashCode(): Int = value.hashCode()
-       
+
   def subArray(start: Int, end: Int): Array[Short] = {
     val result = new Array[Short](end - start)
     Array.copy(value, start, result, 0, end - start)
     result
-  }    
+  }
 
-  def filter(p: Any => Boolean): Array[Short] = {
+  final override def filter(p: Any => Boolean): BoxedArray = {
     val include = new Array[Boolean](value.length)
     var len = 0
     var i = 0
@@ -55,7 +55,13 @@ final class BoxedShortArray(val value: Array[Short]) extends BoxedArray {
       if (include(i)) { result(len) = value(i); len = len + 1 }
       i = i + 1
     }
-    result
+    new BoxedShortArray(result)
+  }
+
+  final override def slice(start: Int, end: Int): BoxedArray = {
+    val len = end - start
+    val result = new Array[Short](len)
+    Array.copy(value, start, result, 0, len)
+    new BoxedShortArray(result)
   }
 }
-
