@@ -3,7 +3,7 @@
  * @author  Martin Odersky
  */
 
-// $Id: Opcodes.scala 8454 2006-08-22 13:38:09Z dragos $
+// $Id: Opcodes.scala 9953 2007-02-09 22:46:09Z dragos $
 
 
 package scala.tools.nsc.backend.icode;
@@ -306,6 +306,18 @@ trait Opcodes requires ICodes {
         else 1
     }
 
+    case class BOX(boxType: TypeKind) extends Instruction {
+      override def toString(): String = "BOX " + boxType
+      override def consumed = 1
+      override def produced = 1
+    }
+
+    case class UNBOX(boxType: TypeKind) extends Instruction {
+      override def toString(): String = "UNBOX " + boxType
+      override def consumed = 1
+      override def produced = 1
+    }
+
     /** Create a new instance of a class through the specified constructor
      * Stack: ...:arg1:arg2:...:argn
      *    ->: ...:ref
@@ -323,9 +335,9 @@ trait Opcodes requires ICodes {
      * Stack: ...:size(int)
      *    ->: ...:arrayref
      */
-    case class CREATE_ARRAY(element: TypeKind) extends Instruction {
+    case class CREATE_ARRAY(elem: TypeKind) extends Instruction {
       /** Returns a string representation of this instruction */
-      override def toString(): String ="CREATE_ARRAY "+element.toString();
+      override def toString(): String ="CREATE_ARRAY "+elem.toString();
 
       override def consumed = 1;
       override def produced = 1;
@@ -497,6 +509,26 @@ trait Opcodes requires ICodes {
       override def consumed = 1;
       override def produced = 0;
     }
+     
+    /** A local variable becomes visible at this point in code. 
+     *  Used only for generating precise local variable tables as
+     *  debugging information.
+     */
+    case class SCOPE_ENTER(lv: Local) extends Instruction {
+      override def toString(): String = "SCOPE_ENTER " + lv
+      override def consumed = 0
+      override def produced = 0
+    }
+
+    /** A local variable leaves its scope at this point in code. 
+     *  Used only for generating precise local variable tables as
+     *  debugging information.
+     */
+    case class SCOPE_EXIT(lv: Local) extends Instruction {
+      override def toString(): String = "SCOPE_EXIT " + lv
+      override def consumed = 0
+      override def produced = 0
+    }
 
     /** This class represents a method invocation style. */
     sealed abstract class InvokeStyle {
