@@ -2,7 +2,7 @@
  * Copyright 2005-2006 LAMP/EPFL
  * @author  Martin Odersky
  */
-// $Id: ConstantFolder.scala 9110 2006-11-01 16:03:28Z mihaylov $
+// $Id: ConstantFolder.scala 10005 2007-02-16 12:02:06Z odersky $
 
 package scala.tools.nsc.typechecker
 
@@ -41,7 +41,7 @@ abstract class ConstantFolder {
   private def fold(tree: Tree, compX: => Constant): Tree =
     try {
       val x = compX
-      if (x != null && x.tag != UnitTag) tree setType ConstantType(x) 
+      if ((x ne null) && x.tag != UnitTag) tree setType mkConstantType(x) 
       else tree
     } catch {
       case _: ArithmeticException => tree   // the code will crash at runtime,
@@ -50,20 +50,20 @@ abstract class ConstantFolder {
     }
 
   private def foldUnop(op: Name, x: Constant): Constant = Pair(op, x.tag) match {
-    case Pair(nme.ZNOT, BooleanTag) => Constant(!x.booleanValue)
+    case Pair(nme.UNARY_!, BooleanTag) => Constant(!x.booleanValue)
 
-    case Pair(nme.NOT , IntTag    ) => Constant(~x.intValue)
-    case Pair(nme.NOT , LongTag   ) => Constant(~x.longValue)
+    case Pair(nme.UNARY_~ , IntTag    ) => Constant(~x.intValue)
+    case Pair(nme.UNARY_~ , LongTag   ) => Constant(~x.longValue)
 
-    case Pair(nme.ADD , IntTag    ) => Constant(+x.intValue)
-    case Pair(nme.ADD , LongTag   ) => Constant(+x.longValue)
-    case Pair(nme.ADD , FloatTag  ) => Constant(+x.floatValue)
-    case Pair(nme.ADD , DoubleTag ) => Constant(+x.doubleValue)
+    case Pair(nme.UNARY_+ , IntTag    ) => Constant(+x.intValue)
+    case Pair(nme.UNARY_+ , LongTag   ) => Constant(+x.longValue)
+    case Pair(nme.UNARY_+ , FloatTag  ) => Constant(+x.floatValue)
+    case Pair(nme.UNARY_+ , DoubleTag ) => Constant(+x.doubleValue)
 
-    case Pair(nme.SUB , IntTag    ) => Constant(-x.intValue)
-    case Pair(nme.SUB , LongTag   ) => Constant(-x.longValue)
-    case Pair(nme.SUB , FloatTag  ) => Constant(-x.floatValue)
-    case Pair(nme.SUB , DoubleTag ) => Constant(-x.doubleValue)
+    case Pair(nme.UNARY_- , IntTag    ) => Constant(-x.intValue)
+    case Pair(nme.UNARY_- , LongTag   ) => Constant(-x.longValue)
+    case Pair(nme.UNARY_- , FloatTag  ) => Constant(-x.floatValue)
+    case Pair(nme.UNARY_- , DoubleTag ) => Constant(-x.doubleValue)
 
     case _ => null
   }
