@@ -2,7 +2,7 @@
  * Copyright 2005-2006 LAMP/EPFL
  * @author  Martin Odersky
  */
-// $Id: Infer.scala 9004 2006-10-20 13:37:17Z odersky $
+// $Id: Infer.scala 10041 2007-02-19 16:34:11Z odersky $
 
 package scala.tools.nsc.typechecker
 
@@ -29,6 +29,9 @@ trait Infer requires Analyzer {
   def assertNonCyclic(tvar: TypeVar) =
     assert(tvar.constr.inst != tvar, tvar.origin)
 
+  def isVarArgs(formals: List[Type]) = 
+    !formals.isEmpty && (formals.last.symbol == RepeatedParamClass)
+
   /** The formal parameter types corresponding to <code>formals</code>.
    *  If <code>formals</code> has a repeated last parameter, a list of 
    *  (nargs - params.length + 1) copies of its type is returned.
@@ -41,12 +44,20 @@ trait Infer requires Analyzer {
       case TypeRef(_, sym, List(arg)) if (sym == ByNameParamClass) => arg
       case formal => formal
     }
-    if (!formals1.isEmpty && (formals1.last.symbol == RepeatedParamClass)) {
+    if (isVarArgs(formals1)) {
       val ft = formals1.last.typeArgs.head
       formals1.init ::: (for (val i <- List.range(formals1.length - 1, nargs)) yield ft)
     } else formals1
   }
 
+  def actualTypes(actuals: List[Type], nformals: int): List[Type] =
+    if (nformals == 1 && actuals.length != 1)
+      List(if (actuals.length == 0) UnitClass.tpe else tupleType(actuals))
+    else actuals
+
+  def actualArgs(pos: PositionType, actuals: List[Tree], nformals: int): List[Tree] =
+    if (nformals == 1 && actuals.length != 1) List(atPos(pos)(gen.mkTuple(actuals))) else actuals
+
   /** A fresh type varable with given type parameter as origin.
    *
    *  @param tparam ...
@@ -104,8 +115,10 @@ trait Infer requires Analyzer {
    *  @param targs   ...
    *  @return        ...
    */
-  private def isWithinBounds(tparams: List[Symbol], targs: List[Type]): boolean = {
-    val bounds = tparams map (.info.subst(tparams, targs).bounds)
+  private def isWithinBounds(pre: Type, owner: Symbol, tparams: List[Symbol], targs: List[Type]): boolean = {
+    val bounds = tparams map { tparam => 
+      tparam.info.asSeenFrom(pre, owner).subst(tparams, targs).bounds
+    }
     !(List.map2(bounds, targs)((bound, targ) => bound containsType targ) contains false)
   }
 
@@ -134,7 +147,7 @@ trait Infer requires Analyzer {
               ((bound contains tparam2) ||
                up && (tparam2.info.bounds.lo =:= tparam.tpe) ||
                !up && (tparam2.info.bounds.hi =:= tparam.tpe))) {
-            if (tvar2.constr.inst == null) cyclic = true
+            if (tvar2.constr.inst eq null) cyclic = true
             solveOne(tvar2, tparam2, variance2)
           }
         }
@@ -159,8 +172,8 @@ trait Infer requires Analyzer {
                   tparam2.tpe.subst(tparams, tvars) :: tvar.constr.lobounds
           }
         }
-        //Console.println("solveOne2 "+tvar+" "+config+" "+tvar.constr.hibounds);//DEBUG
         tvar.constr.inst = NoType // necessary because hibounds/lobounds may contain tvar
+        //Console.println("solving "+tvar+" "+up+" "+(if (up) (tvar.constr.hibounds) else tvar.constr.lobounds))//DEBUG
         tvar.constr.inst = if (up) glb(tvar.constr.hibounds) else lub(tvar.constr.lobounds)
         assertNonCyclic(tvar)//debug
       }
@@ -218,15 +231,15 @@ trait Infer requires Analyzer {
       (if (name.isTypeName) "type " else "value ") + name.decode
 
     def treeSymTypeMsg(tree: Tree): String =
-      if (tree.symbol == null) 
+      if (tree.symbol eq null)
         "expression of type " + tree.tpe
       else if (tree.symbol.hasFlag(OVERLOADED))
         "overloaded method " + tree.symbol + " with alternatives " + tree.tpe
-      else (
+      else
         tree.symbol.toString() +
         (if (tree.tpe.paramSectionCount > 0) ": " else " of type ") +
-        tree.tpe
-      )
+        tree.tpe +
+        (if (tree.symbol.name == nme.apply) tree.symbol.locationString else "")
 
     def applyErrorMsg(tree: Tree, msg: String, argtpes: List[Type], pt: Type) = (
       treeSymTypeMsg(tree) + msg + argtpes.mkString("(", ",", ")") +
@@ -238,6 +251,12 @@ trait Infer requires Analyzer {
         ";\n found   : " + found.toLongString + "\n required: " + req
       }
     
+    def typeErrorMsg(found: Type, req: Type) = 
+      "type mismatch" + foundReqMsg(found, req) +
+      (if (!(found.resultType eq found) && isWeaklyCompatible(found.resultType, req))
+        "\n possible cause: missing arguments for method or constructor"
+       else "")
+      
     def error(pos: PositionType, msg: String): unit =
       context.error(pos, msg)
 
@@ -246,15 +265,12 @@ trait Infer requires Analyzer {
       setError(tree)
     }
 
-    def typeError(pos: PositionType, found: Type, req: Type): unit =
+    def typeError(pos: PositionType, found: Type, req: Type) {
       if (!found.isErroneous && !req.isErroneous) {
-        error(pos,
-          "type mismatch" + foundReqMsg(found, req) +
-             (if (!(found.resultType eq found) && isWeaklyCompatible(found.resultType, req))
-            "\n possible cause: missing arguments for method or constructor"
-           else ""))
+        error(pos, typeErrorMsg(found, req))
         if (settings.explaintypes.value) explainTypes(found, req)
       }
+    }
 
     def typeErrorTree(tree: Tree, found: Type, req: Type): Tree = {
       typeError(tree.pos, found, req)
@@ -269,29 +285,33 @@ trait Infer requires Analyzer {
      */
     private def withDisambiguation[T](tp1: Type, tp2: Type)(op: => T): T = {
 
-      def explainName(sym: Symbol) = { sym.name = newTypeName("<"+sym+" in "+sym.owner+">") }
+      def explainName(sym: Symbol) = { 
+        if (!sym.name.toString.endsWith(")")) {
+          sym.name = newTypeName(sym.name.toString+"(in "+sym.owner+")") 
+        }
+      }
 
       val patches = {
-        val tparams1 = freeTypeParams.collect(tp1)
-        val tparams2 = freeTypeParams.collect(tp2)
+        val syms1 = typeRefs.collect(tp1)
+        val syms2 = typeRefs.collect(tp2)
         for { 
-          val tparam1 <- tparams1
-          val tparam2 <- tparams2
-          tparam1 != tparam2 && tparam1.name == tparam2.name
+          val sym1 <- syms1
+          val sym2 <- syms2
+          sym1 != sym2 && sym1.toString == sym2.toString
         } yield {
-          val name = tparam1.name
-          explainName(tparam1)
-          explainName(tparam2)
-          if (tparam1.owner == tparam2.owner) tparam2.name = newTypeName("some other "+tparam2.name)
-          Triple(tparam1, tparam2, tparam1.name)
+          val name = sym1.name
+          explainName(sym1)
+          explainName(sym2)
+          if (sym1.owner == sym2.owner) sym2.name = newTypeName("(some other)"+sym2.name)
+          Triple(sym1, sym2, name)
         }
       }
 
       val result = op
 
-      for (val Triple(tparam1, tparam2, name) <- patches) {
-        tparam1.name = name
-        tparam2.name = name
+      for (val Triple(sym1, sym2, name) <- patches) {
+        sym1.name = name
+        sym2.name = name
       }
 
       result
@@ -316,24 +336,26 @@ trait Infer requires Analyzer {
           errorTree(tree, underlying(sym).toString() + " cannot be accessed in " +
                     (if (sym.isClassConstructor) context.enclClass.owner else pre.widen) +
                     explanation)
-        sym.toplevelClass match {
+        if (context.unit != null) sym.toplevelClass match {
           case clazz : ClassSymbol =>
             // System.err.println("TOP: " + clazz + " " + clazz.sourceFile)
-            if (clazz.sourceFile != null) 
-              global.currentRun.currentUnit.depends += clazz.sourceFile
+            if (clazz.sourceFile != null) {
+                context.unit.depends += clazz.sourceFile
+              //Console.println("DEPEND " + global.currentRun.currentUnit + " ON " + clazz.sourceFile + " XXX " + context.unit)
+            }
 
           case _ =>
         }
         val sym1 = sym filter (alt => context.isAccessible(alt, pre, site.isInstanceOf[Super]))
         if (sym1 == NoSymbol) {
           if (settings.debug.value) {
-            System.out.println(context)
-            System.out.println(tree)
-            System.out.println("" + pre + " " + sym.owner + " " + context.owner + " " + context.outer.enclClass.owner + " " + sym.owner.thisType + (pre =:= sym.owner.thisType))
+            Console.println(context)
+            Console.println(tree)
+            Console.println("" + pre + " " + sym.owner + " " + context.owner + " " + context.outer.enclClass.owner + " " + sym.owner.thisType + (pre =:= sym.owner.thisType))
           }
           accessError("")
         } else {
-          //System.out.println("check acc " + sym1 + ":" + sym1.tpe + " from " + pre);//DEBUG
+          //Console.println("check acc " + sym1 + ":" + sym1.tpe + " from " + pre);//DEBUG
           var owntype = try{ 
             pre.memberType(sym1)
           } catch {
@@ -407,7 +429,7 @@ trait Infer requires Analyzer {
       /** Map type variable to its instance, or, if `variance' is covariant/contravariant,
        *  to its upper/lower bound */
       def instantiateToBound(tvar: TypeVar, variance: int): Type = try {
-        //System.out.println("instantiate "+tvar+tvar.constr+" variance = "+variance);//DEBUG
+        //Console.println("instantiate "+tvar+tvar.constr+" variance = "+variance);//DEBUG
         if (tvar.constr.inst != NoType) {
           instantiate(tvar.constr.inst)
         } else if ((variance & COVARIANT) != 0 && !tvar.constr.hibounds.isEmpty) {
@@ -456,7 +478,8 @@ trait Infer requires Analyzer {
     *  @return                  ...
     *  @throws                  NoInstance
     */
-    private def methTypeArgs(tparams: List[Symbol], formals: List[Type], restpe: Type, 
+    // bq: was private, but need it for unapply checking
+    def methTypeArgs(tparams: List[Symbol], formals: List[Type], restpe: Type, 
                              argtpes: List[Type], pt: Type,
                              uninstantiated: ListBuffer[Symbol]): List[Type] = {
       val tvars = tparams map freshVar
@@ -504,10 +527,11 @@ trait Infer requires Analyzer {
      *  @return            ...
      */
     def isApplicable(undetparams: List[Symbol], ftpe: Type,
-                     argtpes: List[Type], pt: Type): boolean =
+                     argtpes0: List[Type], pt: Type): boolean =
       ftpe match {
         case MethodType(formals0, restpe) =>
-          val formals = formalTypes(formals0, argtpes.length)
+          val formals = formalTypes(formals0, argtpes0.length)
+          val argtpes = actualTypes(argtpes0, formals.length)
           if (undetparams.isEmpty) {
             (formals.length == argtpes.length &&
              isCompatible(argtpes, formals) && 
@@ -516,18 +540,15 @@ trait Infer requires Analyzer {
             try {
               val uninstantiated = new ListBuffer[Symbol]
               val targs = methTypeArgs(undetparams, formals, restpe, argtpes, pt, uninstantiated)
-              val result = (
-                exprTypeArgs(uninstantiated.toList, restpe.subst(undetparams, targs), pt) != null &&
-                isWithinBounds(undetparams, targs)
-              )
-              result
+              (exprTypeArgs(uninstantiated.toList, restpe.subst(undetparams, targs), pt) ne null) &&
+              isWithinBounds(NoPrefix, NoSymbol, undetparams, targs)
             } catch {
               case ex: NoInstance => false
             }
           }
         case PolyType(tparams, restpe) =>
           val tparams1 = cloneSymbols(tparams)
-          isApplicable(tparams1 ::: undetparams, restpe.substSym(tparams, tparams1), argtpes, pt)
+          isApplicable(tparams1 ::: undetparams, restpe.substSym(tparams, tparams1), argtpes0, pt)
         case ErrorType =>
           true
         case _ =>
@@ -565,12 +586,10 @@ trait Infer requires Analyzer {
     }
 
     /** error if arguments not within bounds. */
-    def checkBounds(pos: PositionType, tparams: List[Symbol],
-                    targs: List[Type], prefix: String): unit =
-      if (!isWithinBounds(tparams, targs)) {
+    def checkBounds(pos: PositionType, pre: Type, owner: Symbol, 
+                    tparams: List[Symbol], targs: List[Type], prefix: String) {
+      if (!isWithinBounds(pre, owner, tparams, targs)) {
         if (!(targs exists (.isErroneous)) && !(tparams exists (.isErroneous))) {
-          //Console.println("tparams = "+tparams+", bounds = "+tparams.map(.info)+", targs="+targs)//DEBUG
-          //withTypesExplained(isWithinBounds(tparams, targs))//DEBUG
           error(pos, 
                 prefix + "type arguments " + targs.mkString("[", ",", "]") + 
                 " do not conform to " + tparams.head.owner + "'s type parameter bounds " + 
@@ -583,6 +602,7 @@ trait Infer requires Analyzer {
           ()
         }
       }
+    }
       
     /** Substitite free type variables `undetparams' of polymorphic argument
      *  expression `tree', given two prototypes `strictPt', and `lenientPt'.
@@ -595,7 +615,7 @@ trait Infer requires Analyzer {
     def inferArgumentInstance(tree: Tree, undetparams: List[Symbol],
                               strictPt: Type, lenientPt: Type): unit = {
       var targs = exprTypeArgs(undetparams, tree.tpe, strictPt)
-      if (targs == null) targs = exprTypeArgs(undetparams, tree.tpe, lenientPt)
+      if (targs eq null) targs = exprTypeArgs(undetparams, tree.tpe, lenientPt)
       substExpr(tree, undetparams, targs, lenientPt)
     }
 
@@ -618,15 +638,15 @@ trait Infer requires Analyzer {
      *  @param pt ...
      */
     private def substExpr(tree: Tree, undetparams: List[Symbol],
-                          targs: List[Type], pt: Type): unit =
-      if (targs == null) {
+                          targs: List[Type], pt: Type) {
+      if (targs eq null) {
         if (!tree.tpe.isErroneous && !pt.isErroneous)
           error(tree.pos, "polymorphic expression cannot be instantiated to expected type" + 
                 foundReqMsg(PolyType(undetparams, skipImplicit(tree.tpe)), pt))
       } else {
-        checkBounds(tree.pos, undetparams, targs, "inferred ")
         new TreeTypeSubstituter(undetparams, targs).traverse(tree)
       }
+    }
 
     /** Substitite free type variables <code>undetparams</code> of application
      *  <code>fn(args)</code>, given prototype <code>pt</code>.
@@ -639,17 +659,18 @@ trait Infer requires Analyzer {
      */
     def inferMethodInstance(fn: Tree, undetparams: List[Symbol],
                             args: List[Tree], pt: Type): List[Symbol] = fn.tpe match {
-      case MethodType(formals, restpe) =>
+      case MethodType(formals0, restpe) =>
         try {
-          val argtpes = args map (.tpe.deconst)
+          val formals = formalTypes(formals0, args.length)
+          val argtpes = actualTypes(args map (.tpe.deconst), formals.length)
           val uninstantiated = new ListBuffer[Symbol]
-          val targs = methTypeArgs(
-            undetparams, formalTypes(formals, argtpes.length),
-            restpe, argtpes, pt, uninstantiated)
-          checkBounds(fn.pos, undetparams, targs, "inferred ")
+          val targs = methTypeArgs(undetparams, formals, restpe, argtpes, pt, uninstantiated)
+          checkBounds(fn.pos, NoPrefix, NoSymbol, undetparams, targs, "inferred ")
+          //Console.println("UNAPPLY subst type "+undetparams+" to "+targs+" in "+fn+" ( "+args+ ")")
           val treeSubst = new TreeTypeSubstituter(undetparams, targs)
           treeSubst.traverse(fn)
           treeSubst.traverseTrees(args)
+          //Console.println("UNAPPLY gives "+fn+" ( "+args+ "), argtpes = "+argtpes+", pt = "+pt)
           uninstantiated.toList
         } catch {
           case ex: NoInstance =>
@@ -713,7 +734,7 @@ trait Infer requires Analyzer {
       def computeArgs =
         try {
           val targs = solve(tvars, undetparams, undetparams map varianceInType(restpe), true)
-          checkBounds(tree.pos, undetparams, targs, "inferred ")
+          checkBounds(tree.pos, NoPrefix, NoSymbol, undetparams, targs, "inferred ")
           new TreeTypeSubstituter(undetparams, targs).traverse(tree)
         } catch {
           case ex: NoInstance => 
@@ -722,7 +743,7 @@ trait Infer requires Analyzer {
                       "\n --- because ---\n" + ex.getMessage())
         }
       def instError = {
-        if (settings.debug.value) System.out.println("ici " + tree + " " + undetparams + " " + pt)
+        if (settings.debug.value) Console.println("ici " + tree + " " + undetparams + " " + pt)
         if (settings.explaintypes.value) explainTypes(restpe.subst(undetparams, tvars), pt)
         errorTree(tree, "constructor cannot be instantiated to expected type" +
                   foundReqMsg(restpe, pt))
@@ -742,13 +763,13 @@ trait Infer requires Analyzer {
           val ptvars = ptparams map freshVar
           val pt1 = pt.subst(ptparams, ptvars)
           if (isPopulated(restpe, pt1)) {
-            ptvars foreach instantiateTypeVar(false)
-          } else { if (settings.debug.value) System.out.println("no instance: "); instError }
-        } else { if (settings.debug.value) System.out.println("not a subtype " + restpe.subst(undetparams, tvars) + " of " + ptWithWildcards); instError }
-      } else { if (settings.debug.value) System.out.println("not fuly defined: " + pt); instError }
+            ptvars foreach instantiateTypeVar
+          } else { if (settings.debug.value) Console.println("no instance: "); instError }
+        } else { if (settings.debug.value) Console.println("not a subtype " + restpe.subst(undetparams, tvars) + " of " + ptWithWildcards); instError }
+      } else { if (settings.debug.value) Console.println("not fuly defined: " + pt); instError }
     }
 
-    def instantiateTypeVar(aliasOK: boolean)(tvar: TypeVar) = {
+    def instantiateTypeVar(tvar: TypeVar) = {
       val tparam = tvar.origin.symbol
       if (false && 
           tvar.constr.inst != NoType && 
@@ -761,17 +782,15 @@ trait Infer requires Analyzer {
       } else {
         val instType = toOrigin(tvar.constr.inst)
         val Pair(loBounds, hiBounds) =
-          if (instType != NoType && isFullyDefined(instType))
-            Pair(List(instType), List(instType))
-          else 
-            Pair(tvar.constr.lobounds, tvar.constr.hibounds)
+          if (instType != NoType && isFullyDefined(instType)) Pair(List(instType), List(instType))
+          else Pair(tvar.constr.lobounds, tvar.constr.hibounds)
         val lo = lub(tparam.info.bounds.lo :: loBounds map toOrigin)
         val hi = glb(tparam.info.bounds.hi :: hiBounds map toOrigin)
         if (!(lo <:< hi)) {
           if (settings.debug.value) log("inconsistent: "+tparam+" "+lo+" "+hi)
         } else if (!((lo <:< tparam.info.bounds.lo) && (tparam.info.bounds.hi <:< hi))) {
           context.nextEnclosing(.tree.isInstanceOf[CaseDef]).pushTypeBounds(tparam)
-          tparam setInfo TypeBounds(lo, hi)
+          tparam setInfo mkTypeBounds(lo, hi)
           if (settings.debug.value) log("new bounds of " + tparam + " = " + tparam.info)
         } else {
           if (settings.debug.value) log("redundant: "+tparam+" "+tparam.info+"/"+lo+" "+hi)
@@ -781,12 +800,12 @@ trait Infer requires Analyzer {
 
     def checkCheckable(pos: PositionType, tp: Type): unit = {
       def patternWarning(tp: Type, prefix: String) =
-        context.unit.deprecationWarning(pos, prefix+tp+" in type pattern is deprecated because it cannot be checked after erasure")
+        context.unit.uncheckedWarning(pos, prefix+tp+" in type pattern is unchecked since it is eliminated by erasure")
       def isLocalBinding(sym: Symbol) =
         sym.isAbstractType && 
         (sym.name == nme.WILDCARD.toTypeName || {
           val e = context.scope.lookupEntry(sym.name)
-          e != null && e.sym == sym && e.owner == context.scope
+          (e ne null) && e.sym == sym && e.owner == context.scope
         })
       tp match {
         case SingleType(pre, _) => 
@@ -803,12 +822,15 @@ trait Infer requires Analyzer {
             }
           }
           checkCheckable(pos, pre)
+        case RefinedType(parents, decls) =>
+          if (decls.isEmpty) for (val p <- parents) checkCheckable(pos, p)
+          else patternWarning(tp, "refinement ")
         case ThisType(_) =>
           ;
         case NoPrefix => 
           ;
         case _ => 
-          patternWarning(tp, "type pattern ")
+          patternWarning(tp, "type ")
       }
     }
 
@@ -829,32 +851,43 @@ trait Infer requires Analyzer {
       }
     }
 
-    def inferTypedPattern(tpt: Tree, pt: Type): Type = {
-      //Console.println("infer typed pattern: "+tpt)//DEBUG
-      checkCheckable(tpt.pos, tpt.tpe)
-      if (!(tpt.tpe <:< pt)) {
-        val tpparams = freeTypeParamsOfTerms.collect(tpt.tpe)
+    def inferTypedPattern(pos: PositionType, pattp: Type, pt: Type): Type = {
+      checkCheckable(pos, pattp)
+      if (!(pattp <:< pt)) {
+        val tpparams = freeTypeParamsOfTerms.collect(pattp)
         if (settings.debug.value) log("free type params (1) = " + tpparams)
         var tvars = tpparams map freshVar
-        var tp = tpt.tpe.subst(tpparams, tvars)
+        var tp = pattp.subst(tpparams, tvars)
         if (!(tp <:< pt)) {
           tvars = tpparams map freshVar
-          tp = tpt.tpe.subst(tpparams, tvars)
+          tp = pattp.subst(tpparams, tvars)
           val ptparams = freeTypeParamsOfTerms.collect(pt)
           if (settings.debug.value) log("free type params (2) = " + ptparams)
           val ptvars = ptparams map freshVar
           val pt1 = pt.subst(ptparams, ptvars)
           if (!isPopulated(tp, pt1)) {
-            error(tpt.pos, "pattern type is incompatibe with expected type"+foundReqMsg(tpt.tpe, pt))
-            return tpt.tpe
+            error(pos, "pattern type is incompatibe with expected type"+foundReqMsg(pattp, pt))
+            return pattp
           }
-          ptvars foreach instantiateTypeVar(false)
+          ptvars foreach instantiateTypeVar
         }
-        tvars foreach instantiateTypeVar(true)
+        tvars foreach instantiateTypeVar
       }
-      intersect(tpt.tpe, pt)
+      intersect(pt, pattp)
     }
 
+    def inferModulePattern(pat: Tree, pt: Type) =
+      if (!(pat.tpe <:< pt)) {
+        val ptparams = freeTypeParamsOfTerms.collect(pt)
+        if (settings.debug.value) log("free type params (2) = " + ptparams)
+        val ptvars = ptparams map freshVar
+        val pt1 = pt.subst(ptparams, ptvars)
+        if (pat.tpe <:< pt1)
+          ptvars foreach instantiateTypeVar
+        else 
+          error(pat.pos, "pattern type is incompatibe with expected type"+foundReqMsg(pat.tpe, pt))
+      }
+
     object toOrigin extends TypeMap {
       def apply(tp: Type): Type = tp match {
         case TypeVar(origin, _) => origin
@@ -862,18 +895,14 @@ trait Infer requires Analyzer {
       }
     }
 
-    abstract class FreeSymCollector extends TypeTraverser {
+    abstract class SymCollector extends TypeTraverser {
       private var result: List[Symbol] = _
       protected def includeCondition(sym: Symbol): boolean
-      private def include(sym: Symbol): unit =
-        if (includeCondition(sym) && !result.contains(sym)) result = sym :: result
 
       override def traverse(tp: Type): TypeTraverser = {
         tp match {
-          case TypeRef(NoPrefix, sym, _) =>
-            include(sym)
-          case TypeRef(ThisType(_), sym, _) =>
-            include(sym)
+          case TypeRef(_, sym, _) =>
+            if (includeCondition(sym) && !result.contains(sym)) result = sym :: result
           case _ =>
         }
         mapOver(tp)
@@ -892,19 +921,36 @@ trait Infer requires Analyzer {
       }
     }
 
+    object approximateAbstracts extends TypeMap {
+      def apply(tp: Type): Type = tp match {
+        case TypeRef(pre, sym, _) if sym.isAbstractType => WildcardType
+        case _ => mapOver(tp)
+      }
+    }
+
     /** A traverser to collect type parameters referred to in a type
      */
-    object freeTypeParamsOfTerms extends FreeSymCollector {
+    object freeTypeParamsOfTerms extends SymCollector {
       protected def includeCondition(sym: Symbol): boolean = sym.isAbstractType && sym.owner.isTerm
     }
-    
-    object freeTypeParams extends FreeSymCollector {
-      protected def includeCondition(sym: Symbol): boolean = 
-        sym.isAbstractType && (sym.owner.isTerm || (sym hasFlag PARAM))
+
+    object typeRefs extends SymCollector {
+      protected def includeCondition(sym: Symbol): boolean = true
     }
 
     /* -- Overload Resolution ---------------------------------------------- */
 
+    def checkNotShadowed(pos: PositionType, pre: Type, best: Symbol, eligible: List[Symbol]) =
+      if (!phase.erasedTypes)
+        for (val alt <- eligible) {
+          if (alt.owner != best.owner && alt.owner.isSubClass(best.owner))
+            error(pos,
+                  "erroneous reference to overloaded definition,\n"+
+                  "most specific definition is: "+best+best.locationString+" of type "+pre.memberType(best)+
+                  ",\nyet alternative definition   "+alt+alt.locationString+" of type "+pre.memberType(alt)+
+                  "\nis defined in a subclass")
+        }
+    
     /** Assign <code>tree</code> the symbol and type of the alternative which
      *  matches prototype <code>pt</code>, if it exists.
      *  If several alternatives match `pt', take parameterless one.
@@ -916,7 +962,6 @@ trait Infer requires Analyzer {
         if (alts1.isEmpty) alts1 = alts
         def improves(sym1: Symbol, sym2: Symbol): boolean =
           sym2 == NoSymbol || 
-          (sym1.owner isSubClass sym2.owner) &&
           { val tp1 = pre.memberType(sym1)
             val tp2 = pre.memberType(sym2)
             (tp2 == ErrorType ||
@@ -929,7 +974,7 @@ trait Infer requires Analyzer {
           if (settings.debug.value) {
             tree match {
               case Select(qual, _) =>
-                System.out.println("qual: " + qual + ":" + qual.tpe +
+                Console.println("qual: " + qual + ":" + qual.tpe +
                                    " with decls " + qual.tpe.decls +
                                    " with members " + qual.tpe.members +
                                    " with members " + qual.tpe.member(newTermName("$minus")))
@@ -942,7 +987,11 @@ trait Infer requires Analyzer {
             context.ambiguousError(tree.pos, pre, best, competing.head, "expected type " + pt)
           setError(tree)
           ()
+          
         } else {
+          val applicable = alts1 filter (alt => 
+            global.typer.infer.isCompatible(pre.memberType(alt), pt))
+          checkNotShadowed(tree.pos, pre, best, applicable)
           tree.setSymbol(best).setType(pre.memberType(best))
         }
       }
@@ -962,8 +1011,7 @@ trait Infer requires Analyzer {
         val applicable = alts filter (alt => isApplicable(undetparams, pre.memberType(alt), argtpes, pt))
         def improves(sym1: Symbol, sym2: Symbol) = (
           sym2 == NoSymbol || sym2.isError ||
-          ((sym1.owner isSubClass sym2.owner) &&
-           specializes(pre.memberType(sym1), pre.memberType(sym2)))
+          specializes(pre.memberType(sym1), pre.memberType(sym2))
         )
         val best = ((NoSymbol: Symbol) /: applicable) ((best, alt) =>
           if (improves(alt, best)) alt else best)
@@ -982,6 +1030,7 @@ trait Infer requires Analyzer {
           setError(tree)
           ()
         } else {
+          checkNotShadowed(tree.pos, pre, best, applicable)
           tree.setSymbol(best).setType(pre.memberType(best))
         }
       }
@@ -1017,23 +1066,43 @@ trait Infer requires Analyzer {
      *  @param tree ...
      *  @param nparams ...
      */
-    def inferPolyAlternatives(tree: Tree, nparams: int): unit = tree.tpe match {
+    def inferPolyAlternatives(tree: Tree, argtypes: List[Type]): unit = tree.tpe match {
       case OverloadedType(pre, alts) =>
-        val sym = tree.symbol filter (alt => alt.typeParams.length == nparams)
-        if (sym == NoSymbol) {
-          errorTree(tree,
-                    if (alts exists (alt => alt.typeParams.length > 0))
-                      "wrong number of type parameters for " + treeSymTypeMsg(tree)
-                    else treeSymTypeMsg(tree) + " does not take type parameters")
-        } else if (sym.hasFlag(OVERLOADED)) {
-          val tparams = sym.alternatives.head.typeParams
-          val tpe = 
-            PolyType(tparams, 
-                     OverloadedType(AntiPolyType(pre, tparams map (.tpe)), sym.alternatives))
-          sym.setInfo(tpe)
-          tree.setSymbol(sym).setType(tpe)
+        val sym0 = tree.symbol filter { alt => alt.typeParams.length == argtypes.length }
+        if (sym0 == NoSymbol) {
+          error(
+            tree.pos,
+            if (alts exists (alt => alt.typeParams.length > 0))
+              "wrong number of type parameters for " + treeSymTypeMsg(tree)
+            else treeSymTypeMsg(tree) + " does not take type parameters")
+          return
+        }
+        if (sym0.hasFlag(OVERLOADED)) {
+          val sym = sym0 filter { alt => isWithinBounds(pre, alt.owner, alt.typeParams, argtypes) }
+          if (sym == NoSymbol) {
+            if (!(argtypes exists (.isErroneous))) {
+              error(
+                tree.pos,
+                "type arguments " + argtypes.mkString("[", ",", "]") + 
+                " conform to the bounds of none of the overloaded alternatives of\n "+sym0+
+                ": "+sym0.info)
+              return
+            }
+          }
+          if (sym.hasFlag(OVERLOADED)) {
+            val tparams = new AsSeenFromMap(pre, sym.alternatives.head.owner).mapOver(
+              sym.alternatives.head.typeParams)
+            val bounds = tparams map (.tpe)
+            val tpe = 
+              PolyType(tparams, 
+                       OverloadedType(AntiPolyType(pre, bounds), sym.alternatives))
+            sym.setInfo(tpe)
+            tree.setSymbol(sym).setType(tpe)
+          } else {
+            tree.setSymbol(sym).setType(pre.memberType(sym))
+          }
         } else {
-          tree.setSymbol(sym).setType(pre.memberType(sym))
+          tree.setSymbol(sym0).setType(pre.memberType(sym0))
         }
     }
   }
