@@ -2,7 +2,7 @@
  * Copyright 2005-2006 LAMP/EPFL
  * @author  Martin Odersky
  */
-// $Id: Namers.scala 9126 2006-11-02 15:12:06Z mcdirmid $
+// $Id: Namers.scala 10048 2007-02-20 13:00:55Z odersky $
 
 package scala.tools.nsc.typechecker
 
@@ -32,7 +32,7 @@ trait Namers requires Analyzer {
           if (tparam == sym || !(tparams contains tparam)) tp
           else rawTypeRef(NoPrefix, tparam, args))
       case SingleType(pre, sym) if (sym.isThisSkolem) => 
-        ThisType(sym.deSkolemize)
+        mkThisType(sym.deSkolemize)
       case PolyType(tparams1, restpe) =>
         new DeSkolemizeMap(tparams1 ::: tparams).mapOver(tp)
       case ClassInfoType(parents, decls, clazz) =>
@@ -83,7 +83,7 @@ trait Namers requires Analyzer {
     def namerOf(sym: Symbol): Namer = {
 
       def innerNamer: Namer = {
-        if (innerNamerCache == null) 
+        if (innerNamerCache eq null)
           innerNamerCache =
             if (!isTemplateContext(context)) this
             else new Namer(context.make(context.tree, context.owner, newScope))
@@ -114,8 +114,8 @@ trait Namers requires Analyzer {
       // allow for overloaded methods
       if (!(sym.isSourceMethod && sym.owner.isClass && !sym.owner.isPackageClass)) {
       	val prev = context.scope.lookupEntry(sym.name);
-      	if (prev != null && prev.owner == context.scope && 
-            (!prev.sym.isSourceMethod || 
+      	if ((prev ne null) && prev.owner == context.scope &&
+            (!prev.sym.isSourceMethod ||
              nme.isSetterName(sym.name) ||
              sym.owner.isPackageClass)) {
      	   doubleDefError(sym.pos, prev.sym)
@@ -154,11 +154,11 @@ trait Namers requires Analyzer {
       if (c.owner.isPackageClass) {
       	val file = context.unit.source.getFile()
       	val clazz = c.asInstanceOf[ClassSymbol]
-      	if (settings.debug.value && clazz.sourceFile != null && !clazz.sourceFile.equals(file)) {
+      	if (settings.debug.value && (clazz.sourceFile ne null) && !clazz.sourceFile.equals(file)) {
           Console.err.println("SOURCE MISMATCH: " + clazz.sourceFile + " vs. " + file + " SYM=" + c);
         }
         clazz.sourceFile = file
-        if (clazz.sourceFile != null) {
+        if (clazz.sourceFile ne null) {
           assert(!currentRun.compiles(clazz) || clazz.sourceFile == currentRun.symSource(c));
           currentRun.symSource(c) = clazz.sourceFile
         }
@@ -253,7 +253,7 @@ trait Namers requires Analyzer {
 	      context.make(tree, tree.symbol.moduleClass, tree.symbol.info.decls))
 	    namer.enterSyms(stats)
 	  case ClassDef(mods, name, tparams, _, impl) =>
-	    if ((mods.flags & (CASE | ABSTRACT)) == CASE) { // enter case factory method.
+	    if ((mods.flags & CASE) != 0) { // enter case factory method.
 	      tree.symbol = enterCaseFactorySymbol(
         		tree.pos, mods.flags & AccessFlags | METHOD | CASE, name.toTermName)
               tree.symbol.setInfo(namerOf(tree.symbol).caseFactoryCompleter(tree))
@@ -269,7 +269,7 @@ trait Namers requires Analyzer {
             tree.symbol.moduleClass.setInfo(namerOf(tree.symbol).moduleClassTypeCompleter(tree))
             finish
           case ValDef(mods, name, tp, rhs) => 
-            if (context.owner.isClass & (mods.flags & LOCAL) == 0) {
+            if (context.owner.isClass && (mods.flags & (PRIVATE | LOCAL)) != (PRIVATE | LOCAL)) {
               val accflags = ACCESSOR |
                 (if ((mods.flags & MUTABLE) != 0) mods.flags & ~MUTABLE else mods.flags | STABLE)
               val getter = owner.newMethod(tree.pos, name).setFlag(accflags)
@@ -335,6 +335,10 @@ trait Namers requires Analyzer {
         if (settings.debug.value) log("defining " + sym);
         val tp = typeSig(tree)
         sym.setInfo(tp)
+        if ((sym.isAliasType || sym.isAbstractType) && !(sym hasFlag PARAM) && 
+            !typer.checkNonCyclic(tree.pos, tp))
+          sym.setInfo(ErrorType) // this early test is there to avoid infinite baseTypes when
+                                 // adding setters and getters --> bug798
         if (settings.debug.value) log("defined " + sym);
         validate(sym)
       }
@@ -393,7 +397,7 @@ trait Namers requires Analyzer {
           .setInfo(typeCompleter(param))
           .setFlag(param.mods.flags & (BYNAMEPARAM | IMPLICIT))
         setPrivateWithin(param, param.symbol, param.mods)
-        context.scope enter param.symbol
+        enterInScope(param.symbol)
         param.symbol
       } else param.symbol
       vparamss.map(.map(enterValueParam))
@@ -428,11 +432,25 @@ trait Namers requires Analyzer {
       ClassInfoType(parents, decls, clazz)    
     }
 
-    private def classSig(tparams: List[AbsTypeDef], tpt: Tree, impl: Template): Type = {
+    private def classSig(tparams: List[AbsTypeDef], self: ValDef, impl: Template): Type = {
+      val clazz = context.owner
       val tparamSyms = typer.reenterTypeParams(tparams)
-      if (!tpt.isEmpty)
-        context.owner.typeOfThis = selfTypeCompleter(tpt)
-      else tpt.tpe = NoType
+      if (!self.tpt.isEmpty) {
+        clazz.typeOfThis = selfTypeCompleter(self.tpt)
+        self.symbol = clazz.thisSym
+      } else {
+        self.tpt.tpe = NoType
+        if (self.name != nme.WILDCARD) {
+          clazz.typeOfThis = clazz.tpe
+          self.symbol = clazz.thisSym
+        } else {
+          self.symbol = clazz.newThisSym(self.pos) setInfo clazz.tpe
+        }
+      }
+      if (self.name != nme.WILDCARD) {
+        context.scope enter self.symbol
+        clazz.thisSym.name = self.name
+      }
       makePolyType(tparamSyms, templateSig(impl))
     }
     
@@ -456,21 +474,23 @@ trait Namers requires Analyzer {
           if (vparamSymss.isEmpty) PolyType(List(), restype)
           else (vparamSymss :\ restype)(makeMethodType))
 
+      var resultPt = if (tpt.isEmpty) WildcardType else typer.typedType(tpt).tpe
+
       if (meth.owner.isClass && (tpt.isEmpty || vparamss.exists(.exists(.tpt.isEmpty)))) {
         // try to complete from matching definition in base type
         for (val vparams <- vparamss; val vparam <- vparams)
           if (vparam.tpt.isEmpty) vparam.symbol setInfo WildcardType
-        val schema = thisMethodType(if (tpt.isEmpty) WildcardType else typer.typedType(tpt).tpe)
+        val schema = thisMethodType(resultPt)
         val site = meth.owner.thisType
         val overridden = intersectionType(meth.owner.info.parents).member(meth.name).filter(sym =>
           sym != NoSymbol && (site.memberType(sym) matches schema))
         if (overridden != NoSymbol && !(overridden hasFlag OVERLOADED)) {
-          var pt = site.memberType(overridden) match {
-            case PolyType(tparams, rt) => rt.substSym(tparamSyms, tparams)
+          resultPt = site.memberType(overridden) match {
+            case PolyType(tparams, rt) => rt.substSym(tparams, tparamSyms)
             case mt => mt
           }
           for (val vparams <- vparamss) {
-            var pfs = pt.paramTypes
+            var pfs = resultPt.paramTypes
             for (val vparam <- vparams) {
               if (vparam.tpt.isEmpty) {
                 vparam.tpt.tpe = pfs.head
@@ -478,17 +498,17 @@ trait Namers requires Analyzer {
               }
               pfs = pfs.tail
             }
-            pt = pt.resultType
+            resultPt = resultPt.resultType
+          }
+          resultPt match {
+            case PolyType(List(), rtpe) => resultPt = rtpe
+            case MethodType(List(), rtpe) => resultPt = rtpe
+            case _ => 
           }
           if (tpt.isEmpty) {
             // provisionally assign `meth' a method type with inherited result type
             // that way, we can leave out the result type even if method is recursive.
-            meth setInfo thisMethodType(
-              pt match {
-                case PolyType(List(), rtpe) => rtpe
-                case MethodType(List(), rtpe) => rtpe
-                case _ => pt
-              })
+            meth setInfo thisMethodType(resultPt)
           }
         }
       }
@@ -500,7 +520,8 @@ trait Namers requires Analyzer {
 
       thisMethodType( 
         if (tpt.isEmpty) {
-          tpt.tpe = deconstIfNotFinal(meth, typer.computeType(rhs)); 
+          val pt = resultPt.substSym(tparamSyms, tparams map (.symbol))
+          tpt.tpe = deconstIfNotFinal(meth, typer.computeType(rhs, WildcardType/*pt*/))
           tpt.tpe
         } else typer.typedType(tpt).tpe)
      }
@@ -529,6 +550,7 @@ trait Namers requires Analyzer {
       var result = tp;
       if (sym hasFlag IMPLICIT) {
         val p = provided(tp);
+        //Console.println("check contractive: "+sym+" "+p+"/"+required(tp))
         for (val r <- required(tp)) {
           if (!isContainedIn(r, p) || (r =:= p)) {
             context.error(sym.pos, "implicit " + sym + " is not contractive," + 
@@ -545,12 +567,25 @@ trait Namers requires Analyzer {
       makePolyType(typer.reenterTypeParams(tparams), typer.typedType(rhs).tpe);
 
     def typeSig(tree: Tree): Type = {
+      val sym: Symbol = tree.symbol
+      tree match {
+        case defn: MemberDef => 
+          val annots = for {
+            val Annotation(constr, elements) <- defn.mods.attributes
+            val ainfo = typer.typedAnnotation(constr, elements)
+            !ainfo.atp.isError
+          } yield ainfo
+          if (!annots.isEmpty) {
+            val annotated = if (sym.isModule) sym.moduleClass else sym
+            annotated.attributes = annots
+          }
+        case _ =>
+      }
       val result = 
         try {
-          val sym: Symbol = tree.symbol
           tree match {
-            case ClassDef(_, _, tparams, tpt, impl) =>
-              new Namer(makeNewScope(context, tree, sym)).classSig(tparams, tpt, impl)
+            case ClassDef(_, _, tparams, self, impl) =>
+              new Namer(makeNewScope(context, tree, sym)).classSig(tparams, self, impl)
             
             case ModuleDef(_, _, impl) =>
               val clazz = sym.moduleClass
@@ -569,7 +604,7 @@ trait Namers requires Analyzer {
         	  context.error(tpt.pos, "missing parameter type");
         	  ErrorType
                 } else { 
-        	  tpt.tpe = deconstIfNotFinal(sym, newTyper(context.make(tree, sym)).computeType(rhs)); 
+        	  tpt.tpe = deconstIfNotFinal(sym, newTyper(context.make(tree, sym)).computeType(rhs, WildcardType)); 
         	  tpt.tpe 
                 }
               } else {
@@ -588,22 +623,22 @@ trait Namers requires Analyzer {
               if (lt.isError) lt = AllClass.tpe
               var ht = typer.typedType(hi).tpe
               if (ht.isError) ht = AnyClass.tpe
-              TypeBounds(lt, ht)
+              mkTypeBounds(lt, ht)
             
             case Import(expr, selectors) =>
               val expr1 = typer.typedQualifier(expr)
               val base = expr1.tpe
               typer.checkStable(expr1)
               def checkNotRedundant(pos: PositionType, from: Name, to: Name): boolean = {
-                if (!tree.symbol.hasFlag(SYNTHETIC) && 
-                    !(expr1.symbol != null && expr1.symbol.isInterpreterWrapper) &&
+                if (!tree.symbol.hasFlag(SYNTHETIC) &&
+                    !((expr1.symbol ne null) && expr1.symbol.isInterpreterWrapper) &&
                     base.member(from) != NoSymbol) {
                   val e = context.scope.lookupEntry(to)
                   def warnRedundant(sym: Symbol) =
                     context.unit.warning(pos, "imported `"+to+
                                          "' is permanently hidden by definition of "+sym+
                                          sym.locationString)
-                  if (e != null && e.owner == context.scope) {
+                  if ((e ne null) && e.owner == context.scope) {
                     warnRedundant(e.sym); return false
                   } else if (context eq context.enclClass) {
                     val defSym = context.prefix.member(to) filter (
@@ -623,7 +658,7 @@ trait Namers requires Analyzer {
                   }
         	  if (from != nme.WILDCARD && (rest.exists (sel => sel._1 == from)))
         	    context.error(tree.pos, from.decode + " is renamed twice");
-        	  if (to != null && to != nme.WILDCARD && (rest exists (sel => sel._2 == to)))
+        	  if ((to ne null) && to != nme.WILDCARD && (rest exists (sel => sel._2 == to)))
         	    context.error(tree.pos, to.decode + " appears twice as a target of a renaming");
         	  checkSelectors(rest)
                 case Nil => 
@@ -685,6 +720,7 @@ trait Namers requires Analyzer {
       checkNoConflict(PRIVATE, PROTECTED)
       checkNoConflict(PRIVATE, OVERRIDE)
       checkNoConflict(DEFERRED, FINAL)
+      checkNoConflict(ABSTRACT, CASE)
     }
   } 
 
